package net.pricefx.common.api;

import java.util.Arrays;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

public enum WorkflowType {
	QUOTE("quote"),
	PRICEGRIDITEM("pricegriditem"),
	REBATEAGREEMENT("rebateagreement"),
	CONTRACT("contract"),
	DATACHANGEREQUEST("dcr"),
	PRICELIST("pricelist"),
	REBATERECORD("rebaterecord"),
	MODELRECORD("modelrecord"),
	CLAIM("claim"),
	DEALPLAN("dealplan");

	private String typeName;

	private WorkflowType(String id) {
		this.typeName = id;
	}
	
	private static final Map<String, WorkflowType> lookupViaDBValue;
	
	static {
		lookupViaDBValue = Arrays.stream(values()).collect(Collectors.toMap(WorkflowType::typeName, Function.identity()));
	}

	@Override
	public String toString() {
		return typeName();
	}

	public String typeName() {
		return typeName;
	}
	
	public static WorkflowType of(String id) {
		for (WorkflowType v : values()) {
			if (v.typeName.equals(id))
				return v;
		}
		throw new IllegalArgumentException("Cannot resolve " + id + " in " + WorkflowType.class.getName());
	}
	
	public static WorkflowType getViaDBValue(String dbValue) {
		return lookupViaDBValue.get(dbValue);
	}
}