package net.pricefx.common.apibuilder.chart;

import java.math.BigDecimal;

import net.pricefx.common.api.chart.ChartType;
import net.pricefx.common.api.chart.LimitPosition;
import net.pricefx.common.api.chart.LimitType;
import net.pricefx.common.api.chart.SortType;
import net.pricefx.common.apibuilder.AbstractAdditionalMeasure;
import net.pricefx.common.apibuilder.AbstractBuilder;
import net.pricefx.common.apibuilder.AbstractOptions;
import net.pricefx.common.apibuilder.AbstractSeries;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.ObjectIdentity;

public class PieChart extends AbstractBuilder {

	public PieChart(String instanceId, String classId, ObjectIdentity parentChartObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
		super(instanceId, classId, idGenerator, invocations);
	}

	private PieOptions options;
	private Dictionary<PieChart> dictionary;
	
	public PieSeries addSeries() {
		String id = addInvocation("addSeries");
		return new PieSeries(this, id, "Series", idGenerator, invocations);
	}
	
	public PieOptions getOptions() {
		String id = addInvocation("getOptions");
		if (options == null) 
			options = new PieOptions(this, id, "Options", idGenerator, invocations);
		return options;
	}
	
	public AuxLine<PieChart> addAuxLine() {
		String id = addInvocation("addAuxLine");
		return new AuxLine<PieChart>(this, id, "AuxLine");
	}
	
	public Dictionary<PieChart> getDictionary() {
		String id = addInvocation("getDictionary");
		if (dictionary == null) 
			dictionary = new Dictionary<>(this, id, "Dictionary");
		return dictionary;
	}
	
	@Override
	public ChartBuilderResult build() {
		addInvocation(METHOD_BUILD);

		ChartBuilderResult r = new ChartBuilderResult();
		r.invocations = invocations.get();
		r.additionalInfo = ChartType.PIE;
		return r;
	}

	
	public static class PieOptions extends AbstractOptions<PieChart,PieOptions> {

		public PieOptions(PieChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		public PieOptions setVerticalLayout(Boolean verticalLayout) {
			
			addInvocation("setVerticalLayout",verticalLayout);
			return this;
		}

	}
	
	public static class PieSeries extends AbstractSeries<PieChart,PieSeries> {

		public PieSeries(PieChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}
		
		public PieSeries setHideDataLabels(Boolean hideDataLabels) {

			addInvocation("setHideDataLabels",hideDataLabels);
			return this;
		}
		
		public LimitToMeasure<PieSeries> setLimitTo(LimitPosition limitPosition, BigDecimal amount, LimitType limitType) {

			String id = addInvocation("setLimitTo",limitPosition, amount, limitType);
			return new LimitToMeasure<PieSeries>(this, id, "LimitToMeasure");
		}
		
		public PieSeries setCategories(String category) {

			addInvocation("setCategories",category);
			return this;
		}

		public Size<PieSeries> setSize(String size) {

			String id = addInvocation("setSize",size);
			return new Size<PieSeries>(this, id, "Size", idGenerator, invocations);
		}

		public PieSeries withSortByCategories(SortType sortType) {

			addInvocation("withSortByCategories",sortType);
			return this;
		}

		public PieSeries withSortBySize(SortType sortType) {

			addInvocation("withSortBySize",sortType);
			return this;
		}
		
		public PieAdditionalMeasure<PieSeries> addAdditionalMeasure() {
			
			String id = addInvocation("addAdditionalMeasure");
			return new PieAdditionalMeasure<PieSeries>(this, id, "AdditionalMeasure", idGenerator, invocations);
		}
		
		public Formula<PieSeries> setFormula() {
			
			String id = addInvocation("setFormula");
			return new Formula<PieSeries>(this, id, "Formula");
		}
		
		public PieSeries setGeneratedQueryDto(String generatedQueryDto) {

			addInvocation("setGeneratedQueryDto",generatedQueryDto);
			return this;
		}
		
	}
	
	public static class Size<ATTACHEDTO extends ObjectIdentity> extends AggrFieldAxis<ATTACHEDTO> {

		public Size(ATTACHEDTO attachedTo, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(attachedTo, instanceId, classId);
		}
		
	}
		
	public static class PieAdditionalMeasure<ATTACHEDTO extends ObjectIdentity> extends AbstractAdditionalMeasure<ATTACHEDTO,PieAdditionalMeasure<ATTACHEDTO>> {
 
		public PieAdditionalMeasure(ATTACHEDTO attachedTo, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, attachedTo, idGenerator, invocations);
		}
		
		public Expression<PieAdditionalMeasure<ATTACHEDTO>> withExpression() { 
		
			String id = addInvocation("withExpression");
			return new Expression<PieAdditionalMeasure<ATTACHEDTO>>(this, id, "Expression");
		}

	}
	
}