package net.pricefx.server.dto;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.Date;

//import net.pricefx.core.i18n.S;



/**
 * This class acts as a DTO for PricelistItems that are enriched by some price list header information
 * As the items are not stored this way it is not a domain object (i.e. no 1:1 table mapping)
 * The extended items will have the same typedid than the regular items though to facilitate easy updates for the manual price
 * (the only editable field)
 *
 */
public class SummaryCalculableObjectItem  {

	/* list header */
	private Date targetDate;
	private Date expiryDate;
	private String listLabel;
	private Date approvalDate;
	
	/* item */	
	private BigInteger version;
	private BigInteger id;
	private String sku;
	private String key2;
	private String typeCode;
	private BigInteger listId;
	private BigDecimal resultPrice;
	private BigDecimal manualResultPrice;
	private BigDecimal previousPrice;
	private BigDecimal priceChange;
	private BigDecimal priceChangePct;
	private Integer approvalState;
	private String comments;
	
		
	public String getComments() {
		return comments;
	}

	public void setComments(String comments) {
		this.comments = comments;
	}

	public String getSku() {
		return sku;
	}

	public void setSku(String sku) {
		this.sku = sku;
	}

	public String getKey2() {
		return key2;
	}

	public void setKey2(String key2) {
		this.key2 = key2;
	}

	public String getTypeCode() {
		return typeCode;
	}

	public void setTypeCode(String typeCode) {
		this.typeCode = typeCode;
	}
	
	public BigInteger getVersion() {
		return version;
	}
	
	public void setVersion(Number version) {		
		if(version instanceof BigInteger)
			this.version = (BigInteger) version;
		else if(version instanceof BigDecimal)
			this.version = ((BigDecimal) version).toBigInteger();		
		else
			throw new RuntimeException("Invalid type:" + version.getClass().getName());
	}
	
	public Date getTargetDate() {
		return targetDate;
	}
	
	public void setTargetDate(Date targetDate) {
		this.targetDate = targetDate;
	}
	
	public Date getExpiryDate() {
		return expiryDate;
	}

	public void setExpiryDate(Date expiryDate) {
		this.expiryDate = expiryDate;
	}
	
	public String getListLabel() {
		return listLabel;
	}
	
	public void setListLabel(String listLabel) {
		this.listLabel = listLabel;
	}
	
	public String getTypedId(){
		return String.format("%s.%s", id,typeCode);
	}
	
	public void setId(Number id) {
		if(id instanceof BigInteger)
			this.id = (BigInteger) id;
		else if(id instanceof BigDecimal)
			this.id = ((BigDecimal) id).toBigInteger();
		else 
			throw new RuntimeException("Invalid type:" + version.getClass().getName());
	}
	
	public BigInteger getListId() {
		return listId;
	}
	
	public void setListId(Number listId) {
		if(listId instanceof BigInteger)
			this.listId = (BigInteger) listId;
		else if(listId instanceof BigDecimal)
			this.listId = ((BigDecimal) listId).toBigInteger();		
		else
			throw new RuntimeException("Invalid type:" + version.getClass().getName());
	}
	
	public BigDecimal getResultPrice() {
		return resultPrice;
	}
	
	public void setResultPrice(BigDecimal resultPrice) {
		this.resultPrice = resultPrice;
	}

	public BigDecimal getManualResultPrice() {
		return manualResultPrice;
	}
	
	public void setManualResultPrice(BigDecimal manualResultPrice) {
		this.manualResultPrice = manualResultPrice;
	}
	
	public BigDecimal getPreviousPrice() {
		return previousPrice;
	}
	
	public void setPreviousPrice(BigDecimal previousPrice) {
		this.previousPrice = previousPrice;
	}
	
	public BigDecimal getPriceChange() {
		return priceChange;
	}
	
	public void setPriceChange(BigDecimal priceChange) {
		this.priceChange = priceChange;
	}
	
	public BigDecimal getPriceChangePct() {
		return priceChangePct;
	}
	
	
	public void setPriceChangePct(BigDecimal priceChangePct) {
		this.priceChangePct = priceChangePct;
	}
	
	public Integer getApprovalState() {
		return approvalState;
	}
	
	
	public void setApprovalState(Number approvalState) {
		if(approvalState instanceof BigDecimal)
			this.approvalState = approvalState.intValue();
		else if(approvalState instanceof Integer)
			this.approvalState = (Integer)approvalState;
		else
			this.approvalState = null;
	}	
			
	public String getApprovalDate() {
		if(approvalDate == null) return null;
		return (new SimpleDateFormat(""/*S.getDateTimePattern()*/)).format(approvalDate);
	}
	
	public void setApprovalDate(Date approvalDate) {
		this.approvalDate = approvalDate;
	}	

	@Override
	public String toString() {
		return String.format("%s [%s]",getTypedId(),getListId());
	}	
	
}
